##############################################################################
#
#    Copyright (C) 2004-2010 Yannick Soldati. All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
from osv.orm import except_orm
from datetime import date

import time
import netsvc


class res_dealers(osv.osv):

    _name = 'res.dealers'
    _description = 'Dealers and Partners'

    _columns = {
        'name':fields.char('Dealer Or Partner', size=64, required=True),
        'partner_id':fields.many2one('res.partner', 'Partner', required=True),
        'parent_id':fields.many2one('res.dealers', 'Parent'),
    }

res_dealers()


class res_users(osv.osv):
    _inherit = 'res.users'

    _columns = {
        'dealers_id': fields.many2one('res.dealers', 'Dealers Or Partners'),
    }

res_users()


class res_partner(osv.osv):
    _inherit = 'res.partner'

    _columns = {
        'dealers_id': fields.many2one('res.dealers', 'Dealers Or Partners'),
    }

    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
            
        current_user = self.pool.get('res.users').read(cr, user, [user])

        if current_user[0]['dealers_id']:
            vals['dealers_id'] = current_user[0]['dealers_id'][0]

        res_id = super(res_partner, self).create(cr, user, vals, context)
        return res_id

res_partner()


class res_partner_address(osv.osv):
    _inherit = 'res.partner.address'

    _columns = {
        'dealers_id': fields.many2one('res.dealers', 'Dealers Or Partners'),
    }

    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
            
        current_user = self.pool.get('res.users').read(cr, user, [user])

        if current_user[0]['dealers_id']:
            vals['dealers_id'] = current_user[0]['dealers_id'][0]

        res_id = super(res_partner_address, self).create(cr, 
                                                         user, vals, context)
        return res_id

res_partner_address()


class crm_case(osv.osv):
    _inherit = 'crm.case'

    def write(self, cr, user, ids, vals, context=None):
        if context is None:
            context = {}
            
        partner_address_obj = self.pool.get('res.partner.address')
        partner_obj = self.pool.get('res.partner')

        if 'user_id' in vals:
            current_user = self.pool.get('res.users').read(cr, user, 
                                                           vals['user_id'])

            if current_user['dealers_id']:
                vals['dealers_id'] = current_user['dealers_id'][0]

                if 'partner_id' in vals:
                    partner = int(vals['partner_id'])
                elif self.read(cr, user, ids[0])['partner_id']:
                    partner = self.read(cr, user, ids[0])['partner_id'][0]
                else:
                    partner = False

                if partner != False:
                    current_partner = partner_obj.write(cr, user, [partner], 
                                          {'dealers_id': 
                                           current_user['dealers_id'][0]})

                    address_ids = partner_address_obj.search(cr, user, 
                                   [('partner_id', 'in', partner_obj.search(
                                                    cr, user, 
                                                    [('id', '=', partner)]))])
                        
                    current_partner_address = partner_address_obj.write(cr, 
                                            user, address_ids, 
                                            {'dealers_id': 
                                            current_user['dealers_id'][0]})

        res_id = super(crm_case, self).write(cr, user, ids, vals, context)
        return res_id


    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
        
        partner_address_obj = self.pool.get('res.partner.address')
        partner_obj = self.pool.get('res.partner')

        if 'user_id' in vals:
            current_user = self.pool.get('res.users').read(cr, user, 
                                                           vals['user_id'])

            if current_user['dealers_id']:
                vals['dealers_id'] = current_user['dealers_id'][0]

                if 'partner_id' in vals:
                    partner = int(vals['partner_id'])
                else:
                    partner = False

                if partner != False:
                    current_partner = partner_obj.write(cr, user, [partner], 
                                            {'dealers_id': 
                                             current_user['dealers_id'][0]})

                    address_ids = partner_address_obj.search(cr, user, 
                                    [('partner_id', 'in', partner_obj.search(
                                                    cr, user, 
                                                    [('id','=',partner)]))])
                    current_partner_address = partner_address_obj.write(cr, 
                                            user, address_ids, 
                                            {'dealers_id': 
                                            current_user['dealers_id'][0]})

        res_id = super(crm_case, self).create(cr, user, vals, context)
        return res_id

    _columns = {
        'dealers_id': fields.many2one('res.dealers', 'Dealers Or Partners'),
    }


crm_case()


class sale_order(osv.osv):
    _inherit = 'sale.order'

    _columns = {
        'dealers_id': fields.many2one('res.dealers', 'Dealers Or Partners'),
        'state': fields.selection([
            ('draft', 'Quotation'),
            ('dealer', "Waiting Art dec'eau validation"),
            ('waiting_date', 'Waiting Schedule'),
            ('manual', 'Manual In Progress'),
            ('progress', 'In Progress'),
            ('shipping_except', 'Shipping Exception'),
            ('invoice_except', 'Invoice Exception'),
            ('done', 'Done'),
            ('cancel', 'Cancelled')
            ], 'Order State', readonly=True, 
                help="Gives the state of the quotation or sale order. \
                The exception state is automatically set when a cancel \
                operation occurs in the invoice validation (Invoice Exception)\
                 or in the packing list process (Shipping Exception). \
                 The 'Waiting Schedule' state is set when the invoice \
                 is confirmed but waiting for the scheduler to run on \
                 the date 'Date Ordered'.", select=True),
    }

    def _make_invoice(self, cr, uid, order, lines, context=None):
        if context is None:
            context = {}
            
        account_id = order.partner_id.property_account_receivable.id
        if order.payment_term:
            pay_term = order.payment_term.id
        else:
            pay_term = False
        for preinv in order.invoice_ids:
            if preinv.state not in ('cancel',):
                for preline in preinv.invoice_line:
                    inv_line_id = self.pool.get('account.invoice.line').copy(
                                    cr, uid, preline.id, 
                                    {'invoice_id': False, 
                                     'price_unit': -preline.price_unit})
                    lines.append(inv_line_id)

        partner_id = order.partner_id.id
        fiscal_position = order.partner_id.property_account_position.id

        if order.origin and order.origin.find('dealer :') != -1:
            partner = order.partner_invoice_id.partner_id
            partner_id = partner.id
            fiscal_position = partner.property_account_position.id

        inv = {
            'name': order.client_order_ref or order.name,
            'origin': order.name,
            'type': 'out_invoice',
            'reference': "P%dSO%d" % (partner_id, order.id),
            'account_id': account_id,
            'partner_id': partner_id,
            'address_invoice_id': order.partner_invoice_id.id,
            'address_contact_id': order.partner_order_id.id,
            'invoice_line': [(6, 0, lines)],
            'currency_id': order.pricelist_id.currency_id.id,
            'comment': order.note,
            'payment_term': pay_term,
            'fiscal_position': fiscal_position,
        }
        inv_obj = self.pool.get('account.invoice')
        inv.update(self._inv_get(cr, uid, order))
        inv_id = inv_obj.create(cr, uid, inv)
        data = inv_obj.onchange_payment_term_date_invoice(cr, uid, 
                                [inv_id], pay_term, time.strftime('%Y-%m-%d'))
        if data.get('value', False):
            inv_obj.write(cr, uid, [inv_id], data['value'], context=context)
        inv_obj.button_compute(cr, uid, [inv_id])
        return inv_id

    def action_cancel(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        order = self.browse(cr, uid, ids[0])

        if order.origin and order.origin.find('dealer :') != -1:

            order_id = order.origin[8:]

            self.write(cr, uid ,int(order_id),{'state': 'cancel'})

        super(sale_order, self).action_cancel(cr, uid, ids, context)

        return True

    def action_wait(self, cr, uid, ids, *args):
        
        self.pool.get('sale.order').write(cr, uid, ids, 
                            {'date_order': date.today().strftime('%Y-%m-%d')})

        order = self.browse(cr, uid, ids[0])

        #When a sale order comes from a dealer the state is 
        #directly set to done
        if order.origin and order.origin.find('dealer :') != -1:

            order_id = order.origin[8:]

            self.write(cr, uid, int(order_id), {'state': 'done'})

        super(sale_order, self).action_wait(cr, uid, ids, *args)

    def action_dealer_confirm(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        order = self.browse(cr, uid, ids[0])

        if order.origin and order.origin.find('dealer :') != -1:
            self.write(cr, uid, ids, {'state': 'dealer'})
        else:
            if ids:
                #copy the sale order

                ad_sale_order = self.copy(cr, uid, ids[0])

                #get the current sale order
                sale_order = self.browse(cr, uid, ids[0])
                if sale_order.dealers_id:
                    address_obj = self.pool.get('res.partner.address')
                    dealer = self.pool.get('res.dealers').read(
                                        cr, uid, [sale_order.dealers_id.id])
                    dealer_partner = self.pool.get('res.partner').read(
                                    cr, uid ,[dealer[0]['partner_id'][0]])

                    #search the dealer address
                    dealer_address = address_obj.search(
                                    cr, uid, [('partner_id', '=', 
                                                dealer[0]['partner_id'][0]), 
                                               ('type', '=', 'invoice')])
                    
                    if dealer_address:
                        pricelist_id = \
                            dealer_partner[0]['property_product_pricelist'][0]
                        self.write(cr, uid, ad_sale_order, {
                            'origin': 'dealer :' + str(sale_order.id),
                            'partner_id': sale_order.partner_id.id,
                            'dealers_id': uid,
                            'partner_invoice_id': dealer_address[0],
                            'pricelist_id': pricelist_id,       
                            'order_policy': 'manual',
                            })

                        self.write(cr, uid, ids, {'state': 'dealer'})

                        order = self.browse(cr, uid, ad_sale_order)

                        orderline_obj = self.pool.get('sale.order.line')

                        for line in order.order_line:
                            # the price is coming for another pricelist
                            data = orderline_obj.product_id_change(
                                cr, uid, line.id, 
                                pricelist_id, 
                                line.product_id.id, line.product_uom_qty, 
                                partner_id=order.partner_id.id)
                        
                            if 'tax_id' in data['value']:
                                del data['value']['tax_id']
                        
                            orderline_obj.write(cr, uid, line.id, 
                                                data['value'])

                        wf_service = netsvc.LocalService("workflow")
                        wf_service.trg_validate(uid, 'sale.order', 
                                                ad_sale_order, 
                                                'dealer_confirm', cr)

                    else:
                        raise except_orm(('Error'), 
                        ('You need to add an invoice address on the dealer'))
        return True

    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
            
        current_user = self.pool.get('res.users').read(cr, user, [user])

        if current_user[0]['dealers_id']:
            vals['dealers_id'] = current_user[0]['dealers_id'][0]

        res_id = super(sale_order, self).create(cr, user, vals, context)
        return res_id

sale_order()


class sale_order_line(osv.osv):
    _inherit = 'sale.order.line'

    _columns = {
        'dealers_id': fields.many2one('res.dealers', 'Dealers Or Partners'),
    }

    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
            
        current_user = self.pool.get('res.users').read(cr, user, [user])

        if current_user[0]['dealers_id']:
            vals['dealers_id'] = current_user[0]['dealers_id'][0]

        res_id = super(sale_order_line, self).create(cr, user, vals, context)
        return res_id

sale_order_line()


class product_pricelist(osv.osv):
    _inherit = 'product.pricelist'

    _columns = {
        'dealers_id': fields.many2one('res.dealers', 'Dealers Or Partners'),
    }

product_pricelist()
